@@ -1,4 +1,4 @@
-/* $Id: main.c 28650 2008-11-07 17:28:08Z nick $ */
+/* $Id$ */
 /*
  *  Copyright (c) 2008 Stephan Arts <stephan@xfce.org>
  *  Copyright (c) 2008 Nick Schermer <nick@xfce.org>
@@ -47,26 +47,36 @@
 
 #include <xfconf/xfconf.h>
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include "accessibility.h"
-#include "displays.h"
 #include "pointers.h"
 #include "keyboards.h"
 #include "keyboard-layout.h"
 #include "keyboard-shortcuts.h"
 #include "workspaces.h"
+#include "clipboard-manager.h"
+#include "utils.h"
+
+#ifdef HAVE_XRANDR
+#include "displays.h"
+#endif
+
+#define SELECTION_NAME  "_XFCE_SETTINGS_HELPER"
+
+static GdkFilterReturn xfce_settings_helper_selection_watcher (GdkXEvent *xevt,
+                                                               GdkEvent *evt,
+                                                               gpointer user_data);
 
 
+static XfceSMClient *sm_client = NULL;
 
 static gboolean opt_version = FALSE;
 static gboolean opt_debug = FALSE;
-static gchar   *opt_sm_client_id = NULL;
 static GOptionEntry option_entries[] =
 {
     { "version", 'V', 0, G_OPTION_ARG_NONE, &opt_version, N_("Version information"), NULL },
     { "debug", 'd', 0, G_OPTION_ARG_NONE, &opt_debug, N_("Start in debug mode (don't fork to the background)"), NULL },
-    { "sm-client-id", 0, 0, G_OPTION_ARG_STRING, &opt_sm_client_id, N_("Client id used when resuming session"), NULL },
     { NULL }
 };
 
@@ -81,12 +91,6 @@ signal_handler (gint signum,
 }
 
 
-static void
-sm_client_die (gpointer client_data)
-{
-    signal_handler (SIGTERM, client_data);
-}
-
 
 static gboolean
 xfce_settings_helper_set_autostart_enabled (gboolean enabled)
@@ -121,144 +125,84 @@ xfce_settings_helper_set_autostart_enabled (gboolean enabled)
 }
 
 
-/* returns TRUE if we're now connected to the SM, FALSE otherwise */
-static gboolean
-xfce_settings_helper_connect_session (int argc,
-                                      char **argv,
-                                      const gchar *sm_client_id)
-{
-    SessionClient *sm_client;
 
-    /* we can't be sure that the SM will save the session later, so we only
-     * disable the autostart item if we're launching because we got *resumed*
-     * from a previous session. */
-
-    sm_client = client_session_new (argc, argv, NULL,
-                                    SESSION_RESTART_IMMEDIATELY, 40);
-    sm_client->die = sm_client_die;
-    if (sm_client_id)
-        client_session_set_client_id (sm_client, sm_client_id);
-    if (!session_init (sm_client))
-    {
-        g_warning ("Failed to connect to session manager");
-        client_session_free (sm_client);
-        xfce_settings_helper_set_autostart_enabled (TRUE);
-        return FALSE;
-    }
-
-    if (sm_client_id && !g_ascii_strcasecmp (sm_client_id, sm_client->given_client_id))
-    {
-        /* we passed a client id, and got the same one back, which means
-         * we were definitely restarted as a part of the session.  so
-         * it's safe to disable the autostart item. */
-        xfce_settings_helper_set_autostart_enabled (FALSE);
-        return TRUE;
-    }
-
-    /* otherwise, let's just ensure the autostart item is enabled. */
-    xfce_settings_helper_set_autostart_enabled (TRUE);
-
-    return TRUE;
-}
-
-
-static gboolean
-xfce_settings_helper_acquire_selection ()
+static GdkFilterReturn
+xfce_settings_helper_selection_watcher (GdkXEvent *xevt,
+                                        GdkEvent *evt,
+                                        gpointer user_data)
 {
 #ifdef GDK_WINDOWING_X11
-    GdkDisplay *gdpy = gdk_display_get_default ();
-    Display *dpy = GDK_DISPLAY_XDISPLAY (gdpy);
-    GdkWindow *rootwin = gdk_screen_get_root_window (gdk_display_get_screen (gdpy, 0));
-    Window xroot = GDK_WINDOW_XID (rootwin);
-    Window xwin;
-    Atom selection_atom, manager_atom;
-    XClientMessageEvent xev;
+    GtkWidget *invisible = GTK_WIDGET (user_data);
+    Window xwin = GDK_WINDOW_XID (invisible->window);
+    XEvent *xe = (XEvent *)xevt;
 
-    xwin = XCreateSimpleWindow (dpy, xroot, -100, -100, 1, 1, 0, 0,
-                                XBlackPixel (GDK_DISPLAY (), 0));
-    XSelectInput (dpy, xwin, PropertyChangeMask | StructureNotifyMask);
-
-    selection_atom = XInternAtom (dpy, "_XFCE_SETTINGS_HELPER", False);
-    manager_atom = XInternAtom (dpy, "MANAGER", False);
-
-    if (XGetSelectionOwner (dpy, selection_atom) != None)
+    if (xe->type == SelectionClear && xe->xclient.window == xwin)
     {
-        XDestroyWindow (dpy, xwin);
-        return FALSE;
+        if (sm_client)
+            xfce_sm_client_set_restart_style (sm_client, XFCE_SM_CLIENT_RESTART_NORMAL);
+        gtk_main_quit ();
     }
-
-    XSetSelectionOwner (dpy, selection_atom, xwin, CurrentTime);
-
-    if (XGetSelectionOwner (dpy, selection_atom) != xwin)
-    {
-        XDestroyWindow (dpy, xwin);
-        return FALSE;
-    }
-
-    xev.type = ClientMessage;
-    xev.window = xroot;
-    xev.message_type = manager_atom;
-    xev.format = 32;
-    xev.data.l[0] = CurrentTime;
-    xev.data.l[1] = selection_atom;
-    xev.data.l[2] = xwin;
-    xev.data.l[3] = xev.data.l[4] = 0;
-
-    XSendEvent (dpy, xroot, False, StructureNotifyMask, (XEvent *)&xev);
 #endif
-
-    return TRUE;
+    return GDK_FILTER_CONTINUE;
 }
 
 
+
 gint
 main (gint argc, gchar **argv)
 {
-    GError     *error = NULL;
-    GObject    *pointer_helper;
-    GObject    *keyboards_helper;
-    GObject    *accessibility_helper;
-    GObject    *shortcuts_helper;
-    GObject    *keyboard_layout_helper;
-    GObject    *displays_helper;
-    GObject    *workspaces_helper;
-    pid_t       pid;
-    guint       i;
-    const gint  signums[] = { SIGHUP, SIGINT, SIGQUIT, SIGTERM };
+    XfceClipboardManager *clipboard_daemon;
+    GError               *error = NULL;
+    GOptionContext       *context;
+    gboolean              in_session;
+    GObject              *pointer_helper;
+    GObject              *keyboards_helper;
+    GObject              *accessibility_helper;
+    GObject              *shortcuts_helper;
+    GObject              *keyboard_layout_helper;
+#ifdef HAVE_XRANDR
+    GObject              *displays_helper;
+#endif
+    GObject              *workspaces_helper;
+    pid_t                 pid;
+    guint                 i;
+    const gint            signums[] = { SIGHUP, SIGINT, SIGQUIT, SIGTERM };
 
     /* setup translation domain */
     xfce_textdomain (GETTEXT_PACKAGE, LOCALEDIR, "UTF-8");
 
-    /* initialize the gthread system */
-    if (!g_thread_supported ())
-        g_thread_init (NULL);
+    /* create option context */
+    context = g_option_context_new (NULL);
+    g_option_context_add_main_entries (context, option_entries, GETTEXT_PACKAGE);
+    g_option_context_add_group (context, gtk_get_option_group (FALSE));
+    g_option_context_add_group (context, xfce_sm_client_get_option_group (argc, argv));
 
     /* initialize gtk */
-    if(!gtk_init_with_args (&argc, &argv, "", option_entries, GETTEXT_PACKAGE, &error))
-    {
-        if (G_LIKELY (error))
-        {
-            /* print error */
-            g_print ("%s: %s.\n", G_LOG_DOMAIN, error->message);
-            g_print (_("Type '%s --help' for usage."), G_LOG_DOMAIN);
-            g_print ("\n");
+    gtk_init (&argc, &argv);
 
-            /* cleanup */
-            g_error_free (error);
-        }
-        else
-        {
-            g_error ("Unable to open display.");
-        }
+    /* parse options */
+    if (!g_option_context_parse (context, &argc, &argv, &error))
+    {
+        /* print error */
+        g_print ("%s: %s.\n", G_LOG_DOMAIN, error->message);
+        g_print (_("Type '%s --help' for usage."), G_LOG_DOMAIN);
+        g_print ("\n");
+
+        /* cleanup */
+        g_error_free (error);
+        g_option_context_free (context);
 
         return EXIT_FAILURE;
     }
 
+    /* cleanup */
+    g_option_context_free (context);
+
     /* check if we should print version information */
     if (G_UNLIKELY (opt_version))
     {
         g_print ("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, PACKAGE_VERSION, xfce_version_string ());
-        g_print ("%s\n", "Copyright (c) 2008");
+        g_print ("%s\n", "Copyright (c) 2008-2010");
         g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
         g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
         g_print ("\n");
@@ -276,12 +220,29 @@ main (gint argc, gchar **argv)
         return EXIT_FAILURE;
     }
 
-    if (!xfce_settings_helper_acquire_selection ())
+    /* connect to session always, even if we quit below.  this way the
+     * session manager won't wait for us to time out. */
+    sm_client = xfce_sm_client_get ();
+    xfce_sm_client_set_restart_style (sm_client, XFCE_SM_CLIENT_RESTART_IMMEDIATELY);
+    g_signal_connect (G_OBJECT (sm_client), "quit", G_CALLBACK (gtk_main_quit), NULL);
+    if (!xfce_sm_client_connect (sm_client, &error) && error)
+    {
+        g_printerr ("Failed to connect to session manager: %s\n", error->message);
+        g_error_free (error);
+    }
+
+    in_session = xfce_sm_client_is_resumed (sm_client);
+    if (!xfce_utils_selection_owner (SELECTION_NAME, in_session,
+                                     xfce_settings_helper_selection_watcher))
     {
         g_printerr ("%s is already running\n", G_LOG_DOMAIN);
+        g_object_unref (G_OBJECT (sm_client));
         return EXIT_FAILURE;
     }
 
+    /* if we were restarted as part of the session, remove us from autostart */
+    xfce_settings_helper_set_autostart_enabled (!in_session);
+
     /* daemonize the process when not running in debug mode */
     if (!opt_debug)
     {
@@ -300,17 +261,25 @@ main (gint argc, gchar **argv)
         }
     }
 
-    xfce_settings_helper_connect_session (argc, argv, opt_sm_client_id);
-
     /* create the sub daemons */
     pointer_helper = g_object_new (XFCE_TYPE_POINTERS_HELPER, NULL);
     keyboards_helper = g_object_new (XFCE_TYPE_KEYBOARDS_HELPER, NULL);
     accessibility_helper = g_object_new (XFCE_TYPE_ACCESSIBILITY_HELPER, NULL);
     shortcuts_helper = g_object_new (XFCE_TYPE_KEYBOARD_SHORTCUTS_HELPER, NULL);
     keyboard_layout_helper = g_object_new (XFCE_TYPE_KEYBOARD_LAYOUT_HELPER, NULL);
+#ifdef HAVE_XRANDR
     displays_helper = g_object_new (XFCE_TYPE_DISPLAYS_HELPER, NULL);
+#endif
     workspaces_helper = g_object_new (XFCE_TYPE_WORKSPACES_HELPER, NULL);
 
+    /* Try to start the clipboard daemon */
+    clipboard_daemon = xfce_clipboard_manager_new ();
+    if (!xfce_clipboard_manager_start (clipboard_daemon))
+    {
+        g_object_unref (G_OBJECT (clipboard_daemon));
+        clipboard_daemon = NULL;
+    }
+
     /* setup signal handlers to properly quit the main loop */
     if (xfce_posix_signal_handler_init (NULL))
     {
@@ -327,11 +296,23 @@ main (gint argc, gchar **argv)
     g_object_unref (G_OBJECT (accessibility_helper));
     g_object_unref (G_OBJECT (shortcuts_helper));
     g_object_unref (G_OBJECT (keyboard_layout_helper));
+#ifdef HAVE_XRANDR
     g_object_unref (G_OBJECT (displays_helper));
+#endif
     g_object_unref (G_OBJECT (workspaces_helper));
 
+    /* Stop the clipboard daemon */
+    if (G_LIKELY (clipboard_daemon != NULL))
+    {
+        xfce_clipboard_manager_stop (clipboard_daemon);
+        g_object_unref (G_OBJECT (clipboard_daemon));
+    }
+
     /* shutdown xfconf */
     xfconf_shutdown ();
 
+    /* release sm client */
+    g_object_unref (G_OBJECT (sm_client));
+
     return EXIT_SUCCESS;
 }
