@@ -35,11 +35,12 @@
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 #include <exo/exo.h>
 #include <xfconf/xfconf.h>
 
 #include "xfce-settings-manager-dialog.h"
+#include "xfce-text-renderer.h"
 
 #ifndef MIN
 #define MIN(a, b)  ( (a) < (b) ? (a) : (b) )
@@ -58,18 +59,21 @@ struct _XfceSettingsManagerDialog
 
     GtkWidget *scrollwin;
 
+    GtkWidget *icon_view;
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     GtkWidget *client_frame;
     GtkWidget *socket_viewport;
     GtkWidget *socket;
 
     GtkWidget *back_button;
+#endif
     GtkWidget *help_button;
 
     const gchar *default_title;
     const gchar *default_subtitle;
     const gchar *default_icon;
 
-    gchar       *help_file;
+    gchar *help_file;
 
     GPid last_pid;
 };
@@ -86,45 +90,46 @@ enum
     COL_COMMENT,
     COL_EXEC,
     COL_SNOTIFY,
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     COL_PLUGGABLE,
     COL_HELP_FILE,
+#endif
+    COL_DIALOG_NAME,
     N_COLS
 };
 
-static void xfce_settings_manager_dialog_class_init(XfceSettingsManagerDialogClass *klass);
-static void xfce_settings_manager_dialog_init(XfceSettingsManagerDialog *dialog);
 static void xfce_settings_manager_dialog_finalize(GObject *obj);
 
-static void xfce_settings_manager_dialog_reset_view(XfceSettingsManagerDialog *dialog,
-                                                    gboolean overview);
 static void xfce_settings_manager_dialog_create_liststore(XfceSettingsManagerDialog *dialog);
-static void xfce_settings_manager_dialog_item_activated(GtkIconView *iconview,
+static void xfce_settings_manager_dialog_item_activated(ExoIconView *iconview,
                                                         GtkTreePath *path,
                                                         gpointer user_data);
-static void xfce_settings_manager_dialog_back_button_clicked(GtkWidget *button,
-                                                             XfceSettingsManagerDialog *dialog);
 static void xfce_settings_manager_dialog_help_button_clicked(GtkWidget *button,
                                                              XfceSettingsManagerDialog *dialog);
 static void xfce_settings_manager_dialog_response(GtkDialog *dialog,
                                                   gint response);
-static void xfce_settings_manager_dialog_plug_added(GtkSocket *socket,
-                                                    XfceSettingsManagerDialog *dialog);
-static gboolean xfce_settings_manager_dialog_plug_removed(GtkSocket *socket,
-                                                          XfceSettingsManagerDialog *dialog);
-static GtkWidget *xfce_settings_manager_dialog_recreate_socket(XfceSettingsManagerDialog *dialog);
 static void xfce_settings_manager_dialog_compute_default_size (XfceSettingsManagerDialog *dialog,
                                                                gint *width,
                                                                gint *height);
 static gboolean xfce_settings_manager_dialog_closed (GtkWidget *dialog,
                                                      GdkEvent *event);
-#if GTK_CHECK_VERSION(2, 12, 0)
+static void xfce_settings_manager_dialog_reset_view(XfceSettingsManagerDialog *dialog,
+                                                    gboolean overview);
+#ifdef ENABLE_PLUGGABLE_DIALOGS
+static void xfce_settings_manager_dialog_back_button_clicked(GtkWidget *button,
+                                                             XfceSettingsManagerDialog *dialog);
+static void xfce_settings_manager_dialog_plug_added(GtkSocket *socket,
+                                                    XfceSettingsManagerDialog *dialog);
+static gboolean xfce_settings_manager_dialog_plug_removed(GtkSocket *socket,
+                                                          XfceSettingsManagerDialog *dialog);
+static GtkWidget *xfce_settings_manager_dialog_recreate_socket(XfceSettingsManagerDialog *dialog);
+#endif
 static gboolean xfce_settings_manager_dialog_query_tooltip(GtkWidget *widget,
                                                            gint x,
                                                            gint y,
                                                            gboolean keyboard_tip,
                                                            GtkTooltip *tooltip,
                                                            gpointer data);
-#endif
 
 
 G_DEFINE_TYPE(XfceSettingsManagerDialog, xfce_settings_manager_dialog, XFCE_TYPE_TITLED_DIALOG)
@@ -142,19 +147,23 @@ static void
 xfce_settings_manager_dialog_init(XfceSettingsManagerDialog *dialog)
 {
     XfconfChannel *channel;
-    GtkWidget *iconview, *scrollwin;
+#ifdef ENABLE_PLUGGABLE_DIALOGS
+    GtkWidget *scrollwin;
+#endif
     GtkCellRenderer *render;
     gint width, height;
 
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     dialog->socket = NULL;
     dialog->last_pid = -1;
 
+    dialog->help_file = NULL;
+#endif
+
     dialog->default_title = _("Settings");
     dialog->default_subtitle = _("Customize your desktop");
     dialog->default_icon = "preferences-desktop";
 
-    dialog->help_file = NULL;
-
     channel = xfconf_channel_get("xfce4-settings-manager");
     xfce_settings_manager_dialog_compute_default_size(dialog, &width, &height);
     width = xfconf_channel_get_int(channel, "/window-width", width);
@@ -165,9 +174,10 @@ xfce_settings_manager_dialog_init(XfceSettingsManagerDialog *dialog)
 
     xfce_titled_dialog_set_subtitle(XFCE_TITLED_DIALOG(dialog),
                                     dialog->default_subtitle);
+    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_window_set_title(GTK_WINDOW(dialog), dialog->default_title);
     gtk_window_set_icon_name(GTK_WINDOW(dialog), dialog->default_icon);
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
+    gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_NORMAL);
 
     dialog->content_frame = gtk_vbox_new(FALSE, 0);
     gtk_widget_show(dialog->content_frame);
@@ -184,44 +194,48 @@ xfce_settings_manager_dialog_init(XfceSettingsManagerDialog *dialog)
     gtk_container_add(GTK_CONTAINER(dialog->content_frame), dialog->scrollwin);
 
     xfce_settings_manager_dialog_create_liststore(dialog);
-    iconview = exo_icon_view_new_with_model(GTK_TREE_MODEL(dialog->ls));
-    exo_icon_view_set_orientation(EXO_ICON_VIEW(iconview),
+    dialog->icon_view = exo_icon_view_new_with_model(GTK_TREE_MODEL(dialog->ls));
+    exo_icon_view_set_orientation(EXO_ICON_VIEW(dialog->icon_view),
                                   GTK_ORIENTATION_HORIZONTAL);
-    exo_icon_view_set_layout_mode(EXO_ICON_VIEW(iconview),
+    exo_icon_view_set_layout_mode(EXO_ICON_VIEW(dialog->icon_view),
                                   EXO_ICON_VIEW_LAYOUT_ROWS);
-    exo_icon_view_set_single_click(EXO_ICON_VIEW(iconview), TRUE);
-    exo_icon_view_set_reorderable(EXO_ICON_VIEW(iconview), FALSE);
-    exo_icon_view_set_selection_mode(EXO_ICON_VIEW(iconview),
-                                     GTK_SELECTION_NONE);
-    gtk_widget_show(iconview);
-    gtk_container_add(GTK_CONTAINER(dialog->scrollwin), iconview);
-    g_signal_connect(G_OBJECT(iconview), "item-activated",
+    exo_icon_view_set_single_click(EXO_ICON_VIEW(dialog->icon_view), TRUE);
+    exo_icon_view_set_reorderable(EXO_ICON_VIEW(dialog->icon_view), FALSE);
+    exo_icon_view_set_selection_mode(EXO_ICON_VIEW(dialog->icon_view),
+                                     GTK_SELECTION_SINGLE);
+    gtk_widget_show(dialog->icon_view);
+    gtk_container_add(GTK_CONTAINER(dialog->scrollwin), dialog->icon_view);
+    g_signal_connect(G_OBJECT(dialog->icon_view), "item-activated",
                      G_CALLBACK(xfce_settings_manager_dialog_item_activated),
                      dialog);
-#if GTK_CHECK_VERSION(2, 12, 0)
-    g_object_set(G_OBJECT(iconview), "has-tooltip", TRUE, NULL);
-    g_signal_connect(G_OBJECT(iconview), "query-tooltip",
+    g_object_set(G_OBJECT(dialog->icon_view), "has-tooltip", TRUE, NULL);
+    g_signal_connect(G_OBJECT(dialog->icon_view), "query-tooltip",
                      G_CALLBACK(xfce_settings_manager_dialog_query_tooltip),
                      NULL);
-#endif
 
     render = gtk_cell_renderer_pixbuf_new();
-    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(iconview), render, FALSE);
-    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(iconview), render,
+    gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(dialog->icon_view), render, 
+                               FALSE);
+    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(dialog->icon_view), render,
                                   "icon-name", COL_ICON_NAME);
-    g_object_set(G_OBJECT(render), "stock-size", GTK_ICON_SIZE_DIALOG, NULL);
+    g_object_set(render, "stock-size", GTK_ICON_SIZE_DIALOG, 
+                 "follow-state", TRUE, NULL);
 
-    render = gtk_cell_renderer_text_new();
-    gtk_cell_layout_pack_end(GTK_CELL_LAYOUT(iconview), render, TRUE);
-    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(iconview), render,
+    render = xfce_text_renderer_new();
+    gtk_cell_layout_pack_end(GTK_CELL_LAYOUT(dialog->icon_view), render, TRUE);
+    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(dialog->icon_view), render,
                                   "text", COL_NAME);
+    g_object_set(render, "follow-state", TRUE, "follow-prelit", TRUE, 
+                 "wrap-width", 128, NULL);
 
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     /* Create client frame to contain the socket scroll window */
     dialog->client_frame = gtk_frame_new (NULL);
     gtk_frame_set_shadow_type(GTK_FRAME(dialog->client_frame), 
                               GTK_SHADOW_NONE);
     gtk_widget_hide(dialog->client_frame);
-    gtk_container_add(GTK_CONTAINER(dialog->content_frame), dialog->client_frame);
+    gtk_container_add(GTK_CONTAINER(dialog->content_frame), 
+                      dialog->client_frame);
 
     /* Create scroll window to contain the socket viewport */
     scrollwin = gtk_scrolled_window_new(NULL, NULL);
@@ -241,6 +255,7 @@ xfce_settings_manager_dialog_init(XfceSettingsManagerDialog *dialog)
 
     /* Create socket */
     dialog->socket = xfce_settings_manager_dialog_recreate_socket(dialog);
+#endif
 
     /* Connect to response signal because maybe we need to kill the settings
      * dialog spawned last before closing the dialog */
@@ -251,8 +266,12 @@ xfce_settings_manager_dialog_init(XfceSettingsManagerDialog *dialog)
     gtk_button_box_set_layout(GTK_BUTTON_BOX(GTK_DIALOG(dialog)->action_area),
                               GTK_BUTTONBOX_EDGE);
 
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     /* Create back button which takes the user back to the overview */
-    dialog->back_button = gtk_button_new_from_stock(GTK_STOCK_GO_BACK);
+    dialog->back_button = gtk_button_new_with_mnemonic(_("_Overview"));
+    gtk_button_set_image(GTK_BUTTON(dialog->back_button),
+                         gtk_image_new_from_stock(GTK_STOCK_GO_BACK, 
+                                                  GTK_ICON_SIZE_BUTTON));
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), 
                       dialog->back_button);
     gtk_widget_set_sensitive(dialog->back_button, FALSE);
@@ -261,6 +280,7 @@ xfce_settings_manager_dialog_init(XfceSettingsManagerDialog *dialog)
     g_signal_connect(dialog->back_button, "clicked", 
                      G_CALLBACK(xfce_settings_manager_dialog_back_button_clicked),
                      dialog);
+#endif
 
     /* Create help button */
     dialog->help_button = gtk_button_new_from_stock(GTK_STOCK_HELP);
@@ -300,20 +320,23 @@ xfce_settings_manager_dialog_reset_view(XfceSettingsManagerDialog *dialog,
         xfce_titled_dialog_set_subtitle(XFCE_TITLED_DIALOG(dialog),
                                         dialog->default_subtitle);
 
+#ifdef ENABLE_PLUGGABLE_DIALOGS
         /* Hide the socket view and display the overview */
         gtk_widget_hide(dialog->client_frame);
         gtk_widget_show(dialog->scrollwin);
 
         /* Hide the back button in the overview */
         gtk_widget_set_sensitive(dialog->back_button, FALSE);
-        
-        /* Show the help button */
-        gtk_widget_show(dialog->help_button);
 
         /* Use default help url */
         g_free(dialog->help_file);
         dialog->help_file = NULL;
+#endif
+        
+        /* Show the help button */
+        gtk_widget_show(dialog->help_button);
     } else {
+#ifdef ENABLE_PLUGGABLE_DIALOGS
         /* Hide overview and (just to be sure) the socket view. The latter is
          * to made visible once a plug has been added to the socket */
         gtk_widget_hide(dialog->scrollwin);
@@ -324,6 +347,7 @@ xfce_settings_manager_dialog_reset_view(XfceSettingsManagerDialog *dialog,
 
         /* Display the back button */
         gtk_widget_set_sensitive(dialog->back_button, TRUE);
+#endif
 
         /* Hide the help button */
         gtk_widget_hide(dialog->help_button);
@@ -362,10 +386,18 @@ xfce_settings_manager_dialog_create_liststore(XfceSettingsManagerDialog *dialog)
 {
     gchar **dirs, buf[PATH_MAX];
     gint i, icon_size;
+    GList *dialog_name_list = NULL;
 
-    dialog->ls = gtk_list_store_new(N_COLS, G_TYPE_STRING, G_TYPE_STRING,
-                                    G_TYPE_STRING, G_TYPE_STRING,
-                                    G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, 
+    dialog->ls = gtk_list_store_new(N_COLS, 
+                                    G_TYPE_STRING, 
+                                    G_TYPE_STRING,
+                                    G_TYPE_STRING, 
+                                    G_TYPE_STRING,
+                                    G_TYPE_BOOLEAN, 
+#ifdef ENABLE_PLUGGABLE_DIALOGS
+                                    G_TYPE_BOOLEAN, 
+                                    G_TYPE_STRING, 
+#endif
                                     G_TYPE_STRING);
     
     dirs = xfce_resource_lookup_all(XFCE_RESOURCE_DATA, "applications/");
@@ -384,7 +416,7 @@ xfce_settings_manager_dialog_create_liststore(XfceSettingsManagerDialog *dialog)
         while((file = g_dir_read_name(d))) {
             XfceRc *rcfile;
             const gchar *name, *exec, *value;
-            gchar **categories;
+            gchar **categories, *dialog_name;
             gboolean have_x_xfce = FALSE, have_desktop_settings = FALSE;
             gint j;
             GtkTreeIter iter;
@@ -457,6 +489,15 @@ xfce_settings_manager_dialog_create_liststore(XfceSettingsManagerDialog *dialog)
                 continue;
             }
 
+            dialog_name = g_strndup(file, g_strrstr(file, ".desktop") - file);
+            /* Make sure we do not store duplicates */
+            if (g_list_find_custom(dialog_name_list, dialog_name, (GCompareFunc)g_utf8_collate)) {
+                xfce_rc_close(rcfile);
+                g_free(dialog_name);
+                continue;
+            }
+
+            dialog_name_list = g_list_prepend (dialog_name_list, dialog_name);
             gtk_list_store_append(dialog->ls, &iter);
             gtk_list_store_set(dialog->ls, &iter,
                                COL_NAME, name,
@@ -464,10 +505,12 @@ xfce_settings_manager_dialog_create_liststore(XfceSettingsManagerDialog *dialog)
                                COL_COMMENT, xfce_rc_read_entry(rcfile, "Comment", NULL),
                                COL_EXEC, exec,
                                COL_SNOTIFY, xfce_rc_read_bool_entry(rcfile, "StartupNotify", FALSE),
+#ifdef ENABLE_PLUGGABLE_DIALOGS
                                COL_PLUGGABLE, xfce_rc_read_bool_entry(rcfile, "X-XfcePluggable", FALSE),
                                COL_HELP_FILE, xfce_rc_read_entry(rcfile, "X-XfceHelpFile", FALSE),
+#endif
+                               COL_DIALOG_NAME, dialog_name,
                                -1);
-
             xfce_rc_close(rcfile);
         }
 
@@ -481,19 +524,24 @@ xfce_settings_manager_dialog_create_liststore(XfceSettingsManagerDialog *dialog)
                                     dialog, NULL);
     gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(dialog->ls),
                                          COL_NAME, GTK_SORT_ASCENDING);
+
+    g_list_foreach(dialog_name_list, (GFunc)g_free, NULL);
+    g_list_free(dialog_name_list);
 }
 
 static void
-xfce_settings_manager_dialog_item_activated(GtkIconView *iconview,
+xfce_settings_manager_dialog_item_activated(ExoIconView *iconview,
                                             GtkTreePath *path,
                                             gpointer user_data)
 {
     XfceSettingsManagerDialog *dialog = user_data;
     GtkTreeIter iter;
-    gchar *exec = NULL, *name, *comment, *icon_name, *primary, *help_file, 
-          *command;
+    gchar *exec = NULL, *name, *comment, *icon_name;
     gboolean snotify = FALSE;
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     gboolean pluggable = FALSE;
+    gchar *help_file, *command;
+#endif
     GError *error = NULL;
 
     if(!gtk_tree_model_get_iter(GTK_TREE_MODEL(dialog->ls), &iter, path))
@@ -505,10 +553,13 @@ xfce_settings_manager_dialog_item_activated(GtkIconView *iconview,
                        COL_EXEC, &exec,
                        COL_ICON_NAME, &icon_name,
                        COL_SNOTIFY, &snotify,
+#ifdef ENABLE_PLUGGABLE_DIALOGS
                        COL_PLUGGABLE, &pluggable,
                        COL_HELP_FILE, &help_file,
+#endif
                        -1);
 
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     /* Kill the previously spawned dialog (if there is any) */
     xfce_settings_manager_dialog_recreate_socket(dialog);
 
@@ -535,48 +586,46 @@ xfce_settings_manager_dialog_item_activated(GtkIconView *iconview,
                                   gtk_socket_get_id(GTK_SOCKET(dialog->socket)));
 
         /* Try to spawn the dialog */
-        if(!xfce_exec_on_screen (gtk_widget_get_screen(GTK_WIDGET(iconview)), 
-                                 command, FALSE, snotify, &error))
+        if(!xfce_spawn_command_line_on_screen(gtk_widget_get_screen(GTK_WIDGET(iconview)),
+                                              exec, FALSE, snotify, &error))
         {
             /* Spawning failed, go back to the overview */
             xfce_settings_manager_dialog_recreate_socket(dialog);
             xfce_settings_manager_dialog_reset_view(dialog, TRUE);
 
             /* Notify the user that there has been a problem */
-            primary = g_strdup_printf(_("Unable to start \"%s\""), exec);
-            xfce_message_dialog(GTK_WINDOW(dialog), _("Xfce Settings Manager"),
-                                GTK_STOCK_DIALOG_ERROR, primary, error->message,
-                                GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
-            g_free(primary);
+            xfce_dialog_show_error(GTK_WINDOW(dialog), error,
+                _("Unable to start \"%s\""), exec);
             g_error_free(error);
         }
 
         g_free(command);
     } else {
+#endif
         /* Switch to the main view (just to be sure) */
         xfce_settings_manager_dialog_reset_view(dialog, TRUE);
 
         /* Try to spawn the dialog */
-        if (!xfce_exec_on_screen(gtk_widget_get_screen(GTK_WIDGET(iconview)),
-                                 exec, FALSE, snotify, &error))
+        if (!xfce_spawn_command_line_on_screen(gtk_widget_get_screen(GTK_WIDGET(iconview)),
+                                               exec, FALSE, snotify, &error))
         {
             /* Notify the user that there has been a problem */
-            primary = g_strdup_printf(_("Unable to start \"%s\""), exec);
-            xfce_message_dialog(GTK_WINDOW(dialog), _("Xfce Settings Manager"),
-                                GTK_STOCK_DIALOG_ERROR, primary, error->message,
-                                GTK_STOCK_CLOSE, GTK_RESPONSE_ACCEPT, NULL);
-            g_free(primary);
+            xfce_dialog_show_error(GTK_WINDOW(dialog), error,
+                _("Unable to start \"%s\""), exec);
             g_error_free(error);
         }
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     }
 
+    g_free(help_file);
+#endif
     g_free(exec);
     g_free(name);
     g_free(comment);
     g_free(icon_name);
-    g_free(help_file);
 }
 
+#ifdef ENABLE_PLUGGABLE_DIALOGS
 static void
 xfce_settings_manager_dialog_back_button_clicked(GtkWidget *button,
                                                  XfceSettingsManagerDialog *dialog)
@@ -585,6 +634,7 @@ xfce_settings_manager_dialog_back_button_clicked(GtkWidget *button,
     xfce_settings_manager_dialog_recreate_socket(dialog);
     xfce_settings_manager_dialog_reset_view(dialog, TRUE);
 }
+#endif
 
 static void 
 xfce_settings_manager_dialog_help_button_clicked(GtkWidget *button,
@@ -608,8 +658,7 @@ xfce_settings_manager_dialog_help_button_clicked(GtkWidget *button,
     if(!gdk_spawn_command_line_on_screen(gtk_widget_get_screen(button), 
                                          command, &error))
     {
-        xfce_err(_("Failed to open the documentation. Reason: %s"), 
-                 error->message);
+        xfce_dialog_show_error (NULL, error, _("Failed to open the documentation"));
         g_error_free(error);
     }
 
@@ -620,21 +669,33 @@ static void
 xfce_settings_manager_dialog_response(GtkDialog *dialog,
                                       gint response)
 {
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     XfceSettingsManagerDialog *sm_dialog = XFCE_SETTINGS_MANAGER_DIALOG(dialog);
+#endif
 
     if(response == GTK_RESPONSE_CLOSE) {
         xfce_settings_manager_dialog_closed(GTK_WIDGET(dialog), NULL);
     }
 
+#ifdef ENABLE_PLUGGABLE_DIALOGS
     /* Make sure the currently embedded dialog is killed before exiting */
     xfce_settings_manager_dialog_recreate_socket(sm_dialog);
+#endif
 }
 
+#ifdef ENABLE_PLUGGABLE_DIALOGS
 static gboolean
 xfce_settings_manager_dialog_show_client(XfceSettingsManagerDialog *dialog)
 {
+    GdkWindow *window;
+    gint width, height;
+
     g_return_val_if_fail(XFCE_IS_SETTINGS_MANAGER_DIALOG(dialog), FALSE);
 
+    window = gtk_socket_get_plug_window(GTK_SOCKET(dialog->socket));
+    gdk_drawable_get_size (GDK_DRAWABLE (window), &width, &height);
+    g_debug ("geometry = (%i,%i)", width, height);
+
     gtk_widget_show(dialog->client_frame);
     return FALSE;
 }
@@ -680,6 +741,7 @@ xfce_settings_manager_dialog_recreate_socket(XfceSettingsManagerDialog *dialog)
 
     return dialog->socket;
 }
+#endif
 
 static void
 xfce_settings_manager_dialog_compute_default_size (XfceSettingsManagerDialog *dialog,
@@ -719,7 +781,8 @@ xfce_settings_manager_dialog_closed (GtkWidget *dialog,
     return FALSE;
 }
 
-#if GTK_CHECK_VERSION(2, 12, 0)
+
+
 static gboolean
 xfce_settings_manager_dialog_query_tooltip(GtkWidget *widget,
                                            gint x,
@@ -755,11 +818,44 @@ xfce_settings_manager_dialog_query_tooltip(GtkWidget *widget,
 
     return TRUE;
 }
-#endif
+
 
 
 GtkWidget *
-xfce_settings_manager_dialog_new()
+xfce_settings_manager_dialog_new(void)
 {
     return g_object_new(XFCE_TYPE_SETTINGS_MANAGER_DIALOG, NULL);
 }
+
+
+
+void 
+xfce_settings_manager_dialog_show_dialog (XfceSettingsManagerDialog *dialog,
+                                          const gchar *dialog_name)
+{
+  GtkTreeModel *model;
+  GtkTreePath *path;
+  GtkTreeIter iter;
+  gchar *name;
+
+  g_return_if_fail (XFCE_IS_SETTINGS_MANAGER_DIALOG (dialog));
+  g_return_if_fail (dialog_name != NULL);
+
+  model = exo_icon_view_get_model(EXO_ICON_VIEW(dialog->icon_view));
+  
+  if(G_LIKELY(gtk_tree_model_get_iter_first(model, &iter))) {
+      do {
+          gtk_tree_model_get(model, &iter, COL_DIALOG_NAME, &name, -1);
+
+          if(G_UNLIKELY(name != NULL && g_str_equal(name, dialog_name))) {
+              path = gtk_tree_model_get_path(model, &iter);
+              xfce_settings_manager_dialog_item_activated(EXO_ICON_VIEW(dialog->icon_view),
+                                                          path, dialog);
+              gtk_tree_path_free(path);
+              break;
+          }
+
+          g_free(name);
+      } while(gtk_tree_model_iter_next(model, &iter));
+  }
+}
